

import spacy
import nltk
from nltk.stem.snowball import SnowballStemmer

import hickle as hkl
import FASTsearch

stemmer = SnowballStemmer("german")


class Passiv2Aktiv(object):
    
    def __init__(self, hklDatabaseDir_Aktiv, hklDatabaseDir_Vorgangspassiv, hklDatabaseDir_Zustandspassiv):
        
        if hklDatabaseDir_Aktiv is not None:
            self.AktivDB = hkl.load(hklDatabaseDir_Aktiv) 
        
        if hklDatabaseDir_Vorgangspassiv is not None:
            self.VorgangspassivDB = hkl.load(hklDatabaseDir_Vorgangspassiv)
            
        if hklDatabaseDir_Zustandspassiv is not None:
            self.ZustandspassivDB = hkl.load(hklDatabaseDir_Zustandspassiv)
        
        
        print('loading the german spacy model..')
        self.nlp = spacy.load('de')
        print('done')
        
        print('loading the stemmer..')
        self.stemmer = SnowballStemmer("german")
        print('done')
        
        return
    
    
    def create_hklDB_from_csv(self, csvDbDir, StemOrNot):
        
        with open(csvDbDir) as lines:
            
            self.DB_All = []
            
            for line in lines:
                
                #print(line)
                
                self.DB_All.append(list(eval(line)))
            
            
            
            
            
            self.hkldb1 = []
            self.hkldb2 = []
            
            
            counter = 0
            for n in range(len(self.DB_All)):
                
                counter += 1
                if counter % 1000 == 0:
                    print(counter)
                
                
                self.hkldb1.append([self.DB_All[n][0][0]] )
                self.hkldb2.append([self.DB_All[n][1][0]] )
                
                
            
            print('creating the hkl dump of DBAll')    
            hkl.dump(self.DB_All, 'hkldb_All' + csvDbDir[:-4] + '.hkl', mode='w', compression='gzip')
            print('done..')
            
            print('Creating the hkl dump of DB 1')
            hkl.dump(self.hkldb1, 'hkldb1' + csvDbDir[:-4] + '.hkl', mode='w', compression='gzip')
            print('done..')
            
            print('Creating the hkl dump of DB 2')
            hkl.dump(self.hkldb2, 'hkldb2' + csvDbDir[:-4] + '.hkl', mode='w', compression='gzip')
            print('done..')
            
        
            
        return 'done'
    
    
    
    def load_DB_into_FASTsearch(self):
        
        self.hkldbAktiv_All = hkl.load('hkldb_AllAktiv.hkl')
        self.hkldbVorgangspassiv_All = hkl.load('hkldb_AllVorgangspassiv.hkl')
        self.hkldbZustandspassiv_All = hkl.load('hkldb_AllZustandspassiv.hkl')
        
        
        
        print('loading hkldbIndi_Conju 1..')
        self.fsearchAktiv1 = FASTsearch.FASTsearch('hkldb1Aktiv.hkl')
        print('done')
        
        print('loading hkldbIndi_Conju 2..')
        self.fsearchAktiv2 = FASTsearch.FASTsearch('hkldb2Aktiv.hkl')
        print('done')
        
        
        # generate bow model only necessary the first time
        #print('generating BoW Model 1..')
        #self.fsearchAktiv1.Gen_BoW_Model(3000, "word", punctuation = False)
        #print('done')
        
        #print('generating BoW Model 2..')
        #self.fsearchAktiv2.Gen_BoW_Model(3000, "word", punctuation = False)
        #print('done')
        
        
        print('loading the bow model 1')
        self.fsearchAktiv1.Load_BoW_Model('bagofwordshkldb1Aktiv.pkl', 'DataBaseOneZeroshkldb1Aktiv.hkl')
        print('done')
        
        print('loading the bow model 2')
        self.fsearchAktiv2.Load_BoW_Model('bagofwordshkldb2Aktiv.pkl', 'DataBaseOneZeroshkldb2Aktiv.hkl')
        print('done')
        
        print('loading hkldbIndi_Conju 1..')
        self.fsearchVorgangspassiv1 = FASTsearch.FASTsearch('hkldb1Vorgangspassiv.hkl')
        print('done')
        
        print('loading hkldbIndi_Conju 2..')
        self.fsearchVorgangspassiv2 = FASTsearch.FASTsearch('hkldb2Vorgangspassiv.hkl')
        print('done')
        
        # uncomment if models are not there
        #print('generating BoW Model 1..')
        #self.fsearchVorgangspassiv1.Gen_BoW_Model(3000, "word", punctuation = False)
        #print('done')
        
        #print('generating BoW Model 2..')
        #self.fsearchVorgangspassiv2.Gen_BoW_Model(3000, "word", punctuation = False)
        #print('done')
        
        
        print('loading the bow model 1')
        self.fsearchVorgangspassiv1.Load_BoW_Model('bagofwordshkldb1Vorgangspassiv.pkl', 'DataBaseOneZeroshkldb1Vorgangspassiv.hkl')
        print('done')
        
        print('loading the bow model 2')
        self.fsearchVorgangspassiv2.Load_BoW_Model('bagofwordshkldb2Vorgangspassiv.pkl', 'DataBaseOneZeroshkldb2Vorgangspassiv.hkl')
        print('done')
        
        print('loading hkldbIndi_Conju 1..')
        self.fsearchZustandspassiv1 = FASTsearch.FASTsearch('hkldb1Zustandspassiv.hkl')
        print('done')
        
        print('loading hkldbIndi_Conju 2..')
        self.fsearchZustandspassiv2 = FASTsearch.FASTsearch('hkldb2Zustandspassiv.hkl')
        print('done')
        
        print('generating BoW Model 1..')
        self.fsearchZustandspassiv1.Gen_BoW_Model(3000, "word", punctuation = False)
        print('done')
        
        print('generating BoW Model 2..')
        self.fsearchZustandspassiv2.Gen_BoW_Model(3000, "word", punctuation = False)
        print('done')
        
        
        print('loading the bow model 1')
        self.fsearchZustandspassiv1.Load_BoW_Model('bagofwordshkldb1Zustandspassiv.pkl', 'DataBaseOneZeroshkldb1Zustandspassiv.hkl')
        print('done')
        
        print('loading the bow model 2')
        self.fsearchZustandspassiv2.Load_BoW_Model('bagofwordshkldb2Zustandspassiv.pkl', 'DataBaseOneZeroshkldb2Zustandspassiv.hkl')
        print('done')
        
        import GS_Utils
        print('initializing the gs utils..')
        self.gs = GS_Utils.GS_Utils('de')
        print('done')
        

        from SentGlue import SentGlueMach
        print('loading the Stochastic Gradient models..')
        self.sgm = SentGlueMach('trainedSGD.pkl', 'bagofwords.pkl')
        print('done')
        print('initializing the SGM..')
        self.sgm.initialize()
        print('done')
        
        #print('oi thats the get_feature_names', self.fsearch1.vectorizer.get_feature_names())
        #print('oi thats the get_feature_names', self.fsearch2.vectorizer.get_feature_names())
        
        
        
    def replacePassivForms(self,sentences):
        
        endsentences = []
        for sentence in sentences:
            
            
            doc = self.nlp(' '.join(sentence))
            
            verbs_of_sentence = []
            wordindex_to_replace = []
            count = 0
            subjectofsentence = []
            subjectindex = []
            erindex = []
            Erindex = []
            undindex = []
            
            for word in doc:
                
                count += 1
                
                #print(word.text)
                #print(word.dep_)
                
                if word.dep_ == 'sb':
                    
                    #print('oi')
                    subjectofsentence.append(word.text)
                    subjectindex.append(count)
                
                if word.text == 'er':
                    erindex.append(count)
                if word.text == 'Er':
                    Erindex.append(count)
                if word.text == 'und':
                    undindex.append(count)
                    
                if word.tag_[0] == 'V':
                    verbs_of_sentence.append(word.text)
                    wordindex_to_replace.append(count)
            
            
            
            verbs_of_sentence_string = ' '.join(verbs_of_sentence)
            length_verbs_of_sentence_string = len(verbs_of_sentence_string)
            
            verbs_of_sentence_string += ' ' + str(length_verbs_of_sentence_string)
            #print(verbs_of_sentence_string)
            bestmatchesZustandspassiv1, matchindexZustandspassiv1 = self.fsearchZustandspassiv1.search_with_highest_multiplikation_Output(verbs_of_sentence_string, 1)    
            
            bestmatchesVorgangspassiv1, matchindexVorgangspassiv1 = self.fsearchVorgangspassiv1.search_with_highest_multiplikation_Output(verbs_of_sentence_string, 1)    
            
            #print(len(verbs_of_sentence))
            #print(matchindexVorgangspassiv1)
            #print(matchindexZustandspassiv1)
            
            vorgangORnot = 0
            zustandORnot = 0
            if (len(verbs_of_sentence) + 1) == matchindexVorgangspassiv1[1]:
                workindex = matchindexVorgangspassiv1[0]
                vorgangORnot = 1
                
            if (len(verbs_of_sentence) + 1) == matchindexZustandspassiv1[1]:
                workindex = matchindexZustandspassiv1[0]
                zustandORnot = 1
           
           
            #print(workindex)
           
            #print(self.hkldbAktiv_All[matchindexVorgangspassiv1[0]])
            #print(self.hkldbVorgangspassiv_All[matchindexVorgangspassiv1[0]])
            #print(self.hkldbZustandspassiv_All[matchindexVorgangspassiv1[0]])
            
            formToReplace = []
            
            if vorgangORnot == 1:
                formToReplace = self.hkldbVorgangspassiv_All[workindex][1][0].split()[-2:]
                formToReplace = '3. Person Singular ' + ' '.join(formToReplace)
                
            
                thrdPersonAktivindex = self.fsearchAktiv2.search_with_highest_multiplikation_Output(formToReplace, 1)[0]
                
                thrdPersonAktiv = self.hkldbAktiv_All[thrdPersonAktivindex[0]][0][0].split()[:-1]
                
                thrdPersonAktiv = ' '.join(thrdPersonAktiv)
                
                for index in reversed(wordindex_to_replace):
                    #print(sentence)
                    #print(index)
                    sentence.pop(index - 1)
                
                thereisasubjectEr = 0
                
                for index in subjectindex:
                    for ind in undindex:
                        if index - 1 == ind:
                            if index - 2 == ('er' or 'Er'):
                                thereisasubjectEr = 1
                        if index + 1 == ind:
                            if index + 2 == ('er' or 'Er'):
                                thereisasubjectEr = 1
                
                for word in subjectofsentence:
                    if word == ('er' or 'Er'):
                        thereisasubjectEr = 1
                
                if thereisasubjectEr == 1:
                    try:
                        sentence.remove('Er')
                    except:
                        sentence.remove('er')
                    sentence.append('ihn')
                
                sentence.append(thrdPersonAktiv)
                sentence.append('jemand')
            
                
                print(sentence)
            
                print('get the tuples and triples to check..')
                tuplesTocheck, triplesTocheck = self.gs.GetTuplesinSentence(sentence)
                print('done')
                print(tuplesTocheck, triplesTocheck)
                
                print('genrating the permutations')
                permutations = self.sgm.GeneratePermutationsOfSentence(sentence)
                print('done')
                print(permutations)
                if (len(tuplesToCheck) != 0) or (len(triplesTocheck) != 0):
                    print('filtering the permutations based on the tuples and triples..')
                    filteredpermutations = self.gs.filterpermutationsaccordingtotuples(permutations, tuplesTocheck, triplesTocheck)
                    print('done')
                else:
                    print('there are no triples or tuples to check..')
                    filteredpermutations = permutations
                    
                sentencesToCheck = []
                for sentence in filteredpermutations:
                    sentencesToCheck.append(' '.join(sentence))
                
                #print('sentencesToCheck', sentencesToCheck)
                print('classifying the probability for right grammar in the filtered permutations..')
                endsentence = self.sgm.GetBestSentenceFromSentencesAccordingToGrammar(sentencesToCheck)
                print('done')
                
                
                endsentences.append(endsentence)
            
            #count1 = 0
            
            #print(subjectindex)
            #subjectindex = subjectindex[0]
            #if subjectindex != 0:
                #for word in sentence[subjectindex - 1:subjectindex + 1]:
                    #count1 += 1
                    #if word == 'und':
                        #thereIsanUnd = count1
            #if subjectindex == 0:
                #for word in sentence[subjectindex:subjectindex + 1]:
                    #count1 += 1
                    #if word == 'und':
                        #thereIsanUnd = count1
            #thereisanEr = 0
            #if sentence[subjectindex - 1 + thereIsanUnd] == 'er' or sentence[subjectindex - 1 + thereIsanUnd] == 'Er':
                
                #thereisanEr = 1
            
             
            #if thereisanEr == 1:
             
                #sentence.remove('Er')
                #sentence.remove('er')
                #sentence.append('ihn')
                
            
            else:
                endsentences.append(sentence)
            
            
            
            
            
            
            
            
            
        
        return(endsentences)
            
            
            
            # Vorgangspassiv wird auf selbe Zeit gemappt, 3. Person Singular.
            # Zustandspassiv: Immer eine Zeit dahinter. D.h.
            # Präsens => Präteritum, Präteritum => Perfekt
            
